"use strict";

this.name           = "TargetSystemPlugins_SharedHandlers";
this.author         = "Nicholas Menchise";
this.copyright      = "(C) 2019 Nicholas Menchise";
this.licence        = "CC BY-NC-SA 4.0";

//
// This world script includes the miscellaneous instructions that are shared
// by the equipment. For example, the code that updates the MFDs. It also
// includes the event handlers.
//



// Whether or not the TBM primeable equipment is processing an activate event. Prevents TMI MFD overload when true.
this.$tbmActive = false;

// Stores the ready status of the TBM primeable equipment.
this.$tbmReady = false;

// Stores the current target mode of the TBM primeable equipment.
this.$tbmTargetMode = 7;    // set to Total mode by default, because it's always displayed on SRS MFD when equipment is activated.

// Calls the function that updates the cascade alert every quarter of a second.
this.$cascadeUpdate = new Timer(this, function() { this._cascadeWeaponAlert(); }, 0, 0.25);

// Stores the SoundSource for the cascade weapon alert. Property set by cascadeWeaponDetected event handler.
this.$cascadeUpdateSound = new SoundSource;

// Whether or not the cascade alert danger text is on the MFD during an alert. It's intended to flash with each timer interval.
this.$cascadeDangerText = true;

// Stores an array of cascade weapon Entities detected by player ship.
this.$cascadeDangerObject = [];


//
// Sets MFD text, if MFDs are available.
//
// This function is called by _statusUpdate and _setMfdString functions in world script TargetSystemPlugins_ShortRangeSnapshot.
// It is also called by the same functions in world script TargetSystemPlugins_TargetMemoryInterface. It is also called by some
// event handlers below. It is also called by _cascadeWeaponMfd function below.
//
// mfdKey argument stores the string that determines where the text will be displayed.
// mfdString argument stores the text to be displayed. It can also set the MFD to blank if the value is null.
//
this._setMfd = function(mfdKey, mfdString) {
    if (player.ship.multiFunctionDisplays > 0) player.ship.setMultiFunctionText(mfdKey, mfdString);
}


//
// Takes over all functioning TSP MFDs to display an alert when a cascade weapon is within the player ship scanner range.
//
// This function is called by $cascadeUpdate timer.
//
this._cascadeWeaponAlert = function() {
    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
    var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];

    // Stops the SRS and TMI timers, if they are running.
    if (tspSRS.$readyUpdate.isRunning) tspSRS.$readyUpdate.stop();
    if (tspTMI.$readyUpdate.isRunning) tspTMI.$readyUpdate.stop();

    // Checks if cascade weapon is still within scanner range and acts accordingly.
    var srsWorking = tspSRS._statusCheck();
    var tmiWorking = tspTMI._statusCheck();
    var youAreHere = player.ship.position;
    var outThere = player.ship.scannerRange;
    if (this.$cascadeDangerObject.length < 1) {
        // Stops the cascade timer and restarts the MFDs, if the equipment is functional.
        this.$cascadeUpdate.stop();
        this.$cascadeDangerText = true;
        if (srsWorking) tspSRS._statusUpdate(true, 3);
        if (tmiWorking) tspTMI._statusUpdate(true, 3);
    }
        else if (!this.$cascadeDangerObject[0].isValid || youAreHere.distanceTo(this.$cascadeDangerObject[0]) > outThere) {
            // Removes the cascade weapon entry from the array.
            var removeCascade = this.$cascadeDangerObject.shift();
        }
            else {
                // Calls the function that sets the string for the MFDs and updates them, if TSP MFDs are available.
                if (srsWorking || tmiWorking) this._cascadeWeaponMfd(srsWorking, tmiWorking, youAreHere);
            }

}


//
// Sets the string for cascade weapon alert, passes the update to the MFDs of functional TSP equipment and plays warning sound.
//
// This function is called by _cascadeWeaponAlert function.
//
// sayThisSRS argument is a boolean that indicates whether or not the SRS is functioning.
// sayThisTMI argument is a boolean that indicates whether or not the TMI is functioning.
// youAreHere argument is the vector position of the player ship.
//
this._cascadeWeaponMfd = function(sayThisSRS, sayThisTMI, youAreHere) {

    // Sets the string for the danger text lines and switches the boolean for the next update.
    var dangerLines = "";
    if (this.$cascadeDangerText) {
        dangerLines = expandDescription("[tsplugins_cascade_danger_text]");
        this.$cascadeDangerText = false;
    }
        else {
            this.$cascadeDangerText = true;
        }

    // Gets the distance between the player ship and the weapon in kilometres.
    var rawDistance = youAreHere.distanceTo(this.$cascadeDangerObject[0]);
    rawDistance /= 1000;
    var fineDistance = "" + rawDistance;
    fineDistance = fineDistance.substring(0, 4);

    // Sets the string for the MFD update.
    var sayThis1 = dangerLines + "\n" + "\n" + "\n";
    var sayThis2 = expandDescription("[tsplugins_cascade_word1]") + "\n" + "\n";
    var sayThis3 = expandDescription("[tsplugins_cascade_word2]") + "\n" + "\n";
    var sayThis4 = "                        " + fineDistance + "km" + "\n" + "\n";
    var sayThis5 = dangerLines;
    var sayThisMfd = sayThis1 + sayThis2 + sayThis3 + sayThis4 + sayThis5;

    // Assigns the string to the functional TSP MFDs and plays the warning sound.
    if (sayThisSRS) this._setMfd(worldScripts[expandDescription("[tsplugins_script_srs]")].$mfdKey, sayThisMfd);
    if (sayThisTMI) this._setMfd(worldScripts[expandDescription("[tsplugins_script_tmi]")].$mfdKey, sayThisMfd);
    this.$cascadeUpdateSound.play();

}


//
//
// Event handlers. See Oolite Javascript Reference pages on Elite Wiki for more details.
//
//


this.startUp = function() {

    // Puts equipment keys in the global array of the sell_equipment world script, if available.
    // The sell_equipment world script is found in the Sell Equipment expansion pack by Commander McLane.
    if (worldScripts["sell_equipment"])
        var x = worldScripts["sell_equipment"].equipment_key.push(
            expandDescription("[tsplugins_eqkey_srs]"), expandDescription("[tsplugins_eqkey_tmi]"),
            expandDescription("[tsplugins_eqkey_tbm]")
        );

    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
    var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];

    // Checks if the equipment is on the player ship and sets the availability status
    if (
        player.ship.equipmentStatus(expandDescription("[tsplugins_eqkey_srs]")) === "EQUIPMENT_OK" ||
        player.ship.equipmentStatus(expandDescription("[tsplugins_eqkey_srs]")) === "EQUIPMENT_DAMAGED"
    ) tspSRS.$onShip = true;
    if (
        player.ship.equipmentStatus(expandDescription("[tsplugins_eqkey_tmi]")) === "EQUIPMENT_OK" ||
        player.ship.equipmentStatus(expandDescription("[tsplugins_eqkey_tmi]")) === "EQUIPMENT_DAMAGED"
    ) tspTMI.$onShip = true;

    // Stops the timers that update the MFDs.
    if (tspSRS.$readyUpdate.isRunning) tspSRS.$readyUpdate.stop();
    if (tspTMI.$readyUpdate.isRunning) tspTMI.$readyUpdate.stop();
    if (this.$cascadeUpdate.isRunning) this.$cascadeUpdate.stop();

    // Initialises the TMI memory bank arrays.
    worldScripts[expandDescription("[tsplugins_script_fs]")]._tmiReset();

}


this.shipWillDockWithStation = function(station) {
    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
    var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];

    // Deactivates the equipment and informs the player on the equipment MFDs, if functional.
    var srsWorking = tspSRS._statusCheck();
    var tmiWorking = tspTMI._statusCheck();
    if (srsWorking) {
        tspSRS.$ready = false;
        tspSRS.$readyUpdate.stop();
        this._setMfd(tspSRS.$mfdKey, expandDescription("[tsplugins_title_srs] [tsplugins_mfd_status_off]"));
    }
        else {
        }
    if (tmiWorking) {
        tspTMI.$ready = false;
        tspTMI.$readyUpdate.stop();
        worldScripts[expandDescription("[tsplugins_script_fs]")]._tmiReset();
        this._setMfd(tspTMI.$mfdKey, expandDescription("[tsplugins_title_tmi] [tsplugins_mfd_status_off]"));
    }
        else {
        }
    this.$tbmReady = false;
    this.$tbmTargetMode = 7;

}


this.shipLaunchedFromStation = function(station) {
    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
    var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];

    // Activates the equipment and informs the player on the equipment MFDs, if functional.
    var srsWorking = tspSRS._statusCheck();
    var tmiWorking = tspTMI._statusCheck();
    if (srsWorking) tspSRS._statusUpdate(true, 3);
    if (tmiWorking) tspTMI._statusUpdate(true, 3);
    if (player.ship.equipmentStatus(expandDescription("[tsplugins_eqkey_tbm]")) === "EQUIPMENT_OK") this.$tbmReady = true;

}


this.shipWillEnterWitchspace = function(cause, destination) {
    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
    var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];

    // Deactivates the equipment temporarily and informs the player on the equipment MFDs, if functional.
    var srsWorking = tspSRS._statusCheck();
    var tmiWorking = tspTMI._statusCheck();
    if (srsWorking) tspSRS._statusUpdate(false, 3);
    if (tmiWorking) {
        tspTMI._statusUpdate(false, 3);
        worldScripts[expandDescription("[tsplugins_script_fs]")]._tmiReset();
    }
        else {
        }
    this.$tbmReady = false;

}


this.shipExitedWitchspace = function() {
    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
    var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];

    // Restores the equipment and informs the player on the equipment MFDs, if functional.
    var srsWorking = tspSRS._statusCheck();
    var tmiWorking = tspTMI._statusCheck();
    if (srsWorking) tspSRS._statusUpdate(true, 3);
    if (tmiWorking) tspTMI._statusUpdate(true, 3);
    if (player.ship.equipmentStatus(expandDescription("[tsplugins_eqkey_tbm]")) === "EQUIPMENT_OK") this.$tbmReady = true;

}


this.equipmentAdded = function(equipmentKey) {

    // Sets the availability status to true for added equipment.
    if (equipmentKey === expandDescription("[tsplugins_eqkey_srs]")) worldScripts[expandDescription("[tsplugins_script_srs]")].$onShip = true;
    if (equipmentKey === expandDescription("[tsplugins_eqkey_tmi]")) worldScripts[expandDescription("[tsplugins_script_tmi]")].$onShip = true;

    if (equipmentKey === expandDescription("[tsplugins_eqkey_tbm]")) this.$tbmReady = true;    // just in case it's added in-flight

}


this.equipmentDamaged = function(equipment) {

    // Informs the player that the equipment is damaged on the equipment MFDs.
    if (equipment === expandDescription("[tsplugins_eqkey_srs]") || equipment === "EQ_SCANNER_SHOW_MISSILE_TARGET")
        var srsWillFail = worldScripts[expandDescription("[tsplugins_script_srs]")]._statusCheck();
    if (equipment === expandDescription("[tsplugins_eqkey_tmi]") || equipment === "EQ_TARGET_MEMORY") {
        var tmiWillFail = worldScripts[expandDescription("[tsplugins_script_tmi]")]._statusCheck();
        worldScripts[expandDescription("[tsplugins_script_fs]")]._tmiReset();
    }
        else {
        }
    if (equipment === expandDescription("[tsplugins_eqkey_tbm]")) this.$tbmReady = false;

}


this.equipmentRemoved = function(equipmentKey) {
    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
    var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];

    // Sets the availability status to false and switches off MFDs for removed equipment.
    if (equipmentKey === expandDescription("[tsplugins_eqkey_srs]")) {
        tspSRS.$onShip = false;
        this._setMfd(tspSRS.$mfdKey, null);
    }
        else {
        }
    if (equipmentKey === expandDescription("[tsplugins_eqkey_tmi]")) {
        tspTMI.$onShip = false;
        this._setMfd(tspTMI.$mfdKey, null);
        worldScripts[expandDescription("[tsplugins_script_fs]")]._tmiReset();
    }
        else {
        }

    if (equipmentKey === expandDescription("[tsplugins_eqkey_tbm]")) this.$tbmReady = false;    // just in case it's removed in-flight

}


this.equipmentRepaired = function(equipment) {
    // This code is for in-flight repairs only, which is possible with some expansion packs.

    // Informs the player if the equipment is restored on the equipment MFDs.
    if (equipment === expandDescription("[tsplugins_eqkey_srs]") || equipment === "EQ_SCANNER_SHOW_MISSILE_TARGET") {
        var srsWillWork = false;
        var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
        if (!player.ship.docked) srsWillWork = tspSRS._statusCheck();
        if (srsWillWork) tspSRS._statusUpdate(true, 3);
    }
        else {
        }
    if (equipment === expandDescription("[tsplugins_eqkey_tmi]") || equipment === "EQ_TARGET_MEMORY") {
        var tmiWillWork = false;
        var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];
        if (!player.ship.docked) tmiWillWork = tspTMI._statusCheck();
        if (tmiWillWork) tspTMI._statusUpdate(true, 3);
    }
        else {
        }

    if (equipment === expandDescription("[tsplugins_eqkey_tbm]") && !player.ship.docked) this.$tbmReady = true;

}


this.playerChangedPrimedEquipment = function(equipmentKey) {
    // Sets TBM to Total mode, because it's a reliable way to indicate that the TBM can access the SRS MFD.
    if (equipmentKey === expandDescription("[tsplugins_eqkey_tbm]")) {
        this.$tbmTargetMode = 7;
        worldScripts[expandDescription("[tsplugins_script_srs]")]._setMfdString();
    }
}


this.cascadeWeaponDetected = function(weapon) {
    // Initiates cascade weapon alert.
    var putThatHere = this.$cascadeDangerObject.push(weapon);
    this.$cascadeUpdateSound.sound = "warning.ogg";
    if (!this.$cascadeUpdate.isRunning) this.$cascadeUpdate.start();
}


this.shipTargetAcquired = function(target) {
    // Passes new target to the TMI, if functional.
    // Wormholes are not passed to the TMI because the Target System Memory Expansion ignores them.
    // The functions to update the TMI MFD are called by the TBM when it's active, hence the condition at the end,
    // which prevents excessive updates to the MFD.
    //
    var tspTMI = worldScripts[expandDescription("[tsplugins_script_tmi]")];
    var tspFS = worldScripts[expandDescription("[tsplugins_script_fs]")];
    var tmiWorking = tspTMI._statusCheck();
    if (tmiWorking && !target.isWormhole) tspFS._tmiBankThisOrNot(target);
    if (tmiWorking && !this.$tbmActive) {
        tspFS._tmiGroupInRange();
        tspTMI._setMfdString();
    }
        else {
        }
}

