"use strict";

this.name           = "TargetSystemPlugins_TargetMemoryInterface";
this.author         = "Nicholas Menchise";
this.copyright      = "(C) 2019 Nicholas Menchise";
this.licence        = "CC BY-NC-SA 4.0";

//
// This world script includes the instructions for the TMI MFD strings and checking the equipment status.
//


// Whether or not the equipment is on the player ship
this.$onShip = false;

// Whether or not the equipment is functioning
this.$ready = false;

// Whether or not MFD updates should be suspended to prevent tip strings from changing every second
this.$mfdSuspend = false;

// Calls the function that updates the equipment MFD at regular intervals, if the equipment is functioning
this.$readyUpdate = new Timer(this, function() { this._timedUpdate(); }, 0, 1);

// Stores the MFD key for the equipment
this.$mfdKey = "tsplugins_tmi";


//
// Checks the status of the equipment and its dependencies. Returns a boolean.
//
// This function is called by _timedUpdate function. It is also called by _setMfdString function in
// world script TargetSystemPlugins_ShortRangeSnapshot. It is also called by _statusCheck function
// in equipment script Target SystemPlugins_TargetBankingModule. It is also called by event handlers
// in world script TargetSystemPlugins_SharedHandlers. It is also called by _cascadeWeaponAlert function
// in world script TargetSystemPlugins_SharedHandlers.
//
// A negative status (false) will trigger a function call that results in a status update.
// An integer is calculated and passed in the same function call to provide more specific
// information in the status update.
//
this._statusCheck = function() {
    var dependsValue = 0;
    var dependsResult = false;

    if (player.ship.equipmentStatus(expandDescription("[tsplugins_eqkey_tmi]")) === "EQUIPMENT_OK") dependsValue = 2;
    if (player.ship.equipmentStatus("EQ_TARGET_MEMORY") === "EQUIPMENT_OK") dependsValue++;

    if (dependsValue == 3) dependsResult = true;
    if (this.$ready && dependsValue < 3) this._statusUpdate(false, dependsValue);

    return dependsResult;
}


//
// Updates the status of the equipment and informs the player when it's activated, disabled or damaged.
//
// This function is called by _statusCheck function. It is also called by event handlers in the world
// script TargetSystemPlugins_SharedHandlers.
//
// isWorking argument stores a boolean that indicates if the equipment is enabled.
//
// whatHappened argument stores an integer that indicates the cause of the status change.
// A value of 3 means that the equipment is fine, but something else may be stopping it, e.g. witchspace tunnel.
// A value of 2 means that the dependency check has failed.
// A value less than 2 means that the equipment is damaged.
//
this._statusUpdate = function(isWorking, whatHappened) {

    // Changes the ready status and stops the timer, if necessary. Stops the function if equipment is off the ship.
    if (this.$ready !== isWorking) this.$ready = isWorking;
    if (!isWorking && this.$readyUpdate.isRunning) this.$readyUpdate.stop();
    if (!this.$onShip) return;

    // Sets the string for the status update to the MFD.
    var theseLines = "";    // stores the MFD update string.
    if (isWorking) theseLines = expandDescription("[tsplugins_ready_tmi]");
    if (!isWorking && whatHappened == 3) theseLines = expandDescription("[tsplugins_title_tmi][tsplugins_mfd_status_witchspace]");
    if (!isWorking && whatHappened == 2) theseLines = expandDescription("[tsplugins_disabled1_tmi]");
    if (!isWorking && whatHappened < 2) theseLines = expandDescription("[tsplugins_broken_tmi]");

    // Calls the MFD setting function.
    worldScripts[expandDescription("[tsplugins_script_sh]")]._setMfd(this.$mfdKey, theseLines);

    // Resumes the timer that calls regular updates to the MFD, if the equipment has become functional.
    // The two second delay ensures that the player can see the status update when the equipment is activated.
    //
    if (isWorking && !this.$readyUpdate.isRunning) {
        this.$readyUpdate.nextTime = clock.absoluteSeconds + 2;
        this.$readyUpdate.start();
        this.$mfdSuspend = false;    // fixed a bug where the MFD was stuck on startup instead of displaying tip
    }
        else {
        }
}


//
// Updates the equipment MFD at regular intervals, if the equipment is functional and targets are within scanner range.
//
// This function is called by $readyUpdate timer.
//
this._timedUpdate = function() {
    var isWorking = this._statusCheck();
    if (!isWorking) return;
    var tspFS = worldScripts[expandDescription("[tsplugins_script_fs]")];
    tspFS._tmiGroupInRange();
    if (tspFS.$targetsMemoryInRange.length > 0) this.$mfdSuspend = false;
    var srsWorking = worldScripts[expandDescription("[tsplugins_script_srs]")]._statusCheck();
    if (srsWorking) tspFS._tmiSignUpdate();
    if (!this.$mfdSuspend) this._setMfdString();
}


//
// Sets the string that will be sent to the equipment MFD and calls the MFD update.
//
// This function is called by _timedUpdate function.
// It is also called by shipTargetAcquired event handler in world script TargetSystemPlugins_SharedHandlers.
// It is also called by .activated event handler in equipment script TargetSystemPlugins_TargetBankingModule.
//
// This should only be called shortly after _tmiGroupInRange function in world script
// TargetSystemPlugins_FilterSort is called, if not immediately afterwards.
//
this._setMfdString = function() {
    var sayThisMfd = "";
    var srsWorking = worldScripts[expandDescription("[tsplugins_script_srs]")]._statusCheck();
    var tspFS = worldScripts[expandDescription("[tsplugins_script_fs]")];
    var targetsIndex = tspFS.$targetsMemoryInRange;

    if (targetsIndex.length > 0) {
    // Sets the target list string when targets are found.

        // Sets the first line, which includes the title and number of targets in memory that are within scanner range.
        // The second line is blank.
        //
        var lineOne = expandDescription("[tsplugins_title_tmi]") + "                " + targetsIndex.length + "\n" + "\n";

        // Gets the columns for the remaining lines.
        var selectColumn = this._getSelectionColumn(tspFS.$targetsMemoryData, targetsIndex);
        var symbolColumn = this._getCategoryColumn(tspFS.$targetsMemorySigns, targetsIndex, srsWorking);
        var namesColumn = this._getTargetNamesColumn(tspFS.$targetsMemoryData, targetsIndex);

        // Sets up the columns.
        var linesEight = "";
        for (var n = 0; n < 8; n++) {
            linesEight = linesEight + selectColumn[n] + symbolColumn[n] + "  " + namesColumn[n];
            if (n < 7) linesEight += "\n";
        }

        sayThisMfd = lineOne + linesEight;
    }
        else if (srsWorking) {
            // Sets the TMI Guide string when no targets are found. The type of string depends on the SRS status.
            sayThisMfd = expandDescription("[tsplugins_mfd_tip1_tmi]");
            this.$mfdSuspend = true;
        }
            else {
                sayThisMfd = expandDescription("[tsplugins_mfd_tip2_tmi]");
                this.$mfdSuspend = true;
            }

    // Calls the MFD setting function.
    worldScripts[expandDescription("[tsplugins_script_sh]")]._setMfd(this.$mfdKey, sayThisMfd);
}


//
// Returns an array of strings that define the target selection column of the TMI MFD.
//
// This function is called by _setMfdString function.
//
// targetsInHistory argument is an array of objects selected with the player ship's targeting system recently.
// targetsInRange argument is an array of index positions of objects in targetsInHistory that are within the
// player ship's scanner range.
//
this._getSelectionColumn = function(targetsInHistory, targetsInRange) {
    var thisColumn = [];
    for (var n = 0; n < 8; n++) thisColumn[n] = expandDescription("[tsplugins_mfd_unselected]");
    for (var n = 0; n < targetsInRange.length; n++) {
        var thisOne = targetsInRange[n];
        if (targetsInHistory[thisOne] === player.ship.target)
            thisColumn[n] = expandDescription("[tsplugins_mfd_selected]");
    }
    return thisColumn;
}


//
// Returns an array of strings that define the target category column of the TMI MFD.
//
// This function is called by _setMfdString function.
//
// symbolsInHistory argument is an array of target category symbols corresponding to objects selected
// with the player ship's targeting system recently.
// targetsInRange argument is an array of index positions of symbolsInHistory corresponding to objects
// that are within the player ship's scanner range.
// snapshotAvailable argument is a boolean that indicates whether or not the SRS is functioning.
//
this._getCategoryColumn = function(symbolsInHistory, targetsInRange, snapshotAvailable) {
    var thisColumn = [];
    for (var n = 0; n < 8; n++) thisColumn[n] = expandDescription("[tsplugins_sign_blank]");
    for (var n = 0; n < targetsInRange.length; n++) {
        if (!snapshotAvailable) break;
        var thisOne = targetsInRange[n];
        thisColumn[n] = symbolsInHistory[thisOne];
    }
    return thisColumn;
}


//
// Returns an array of strings that define the target names column of the TMI MFD.
//
// This function is called by _setMfdString function.
//
// targetsInHistory argument is an array of objects selected with the player ship's targeting system recently.
// targetsInRange argument is an array of index positions of objects in targetsInHistory that are within the
// player ship's scanner range.
//
this._getTargetNamesColumn = function(targetsInHistory, targetsInRange) {
    var thisColumn = [];
    for (var n = 0; n < 8; n++) thisColumn[n] = "";
    for (var n = 0; n < targetsInRange.length; n++) {
        var thisOne = targetsInRange[n];
        var nameThisOne = targetsInHistory[thisOne].displayName;
        if (nameThisOne.length > 31) nameThisOne = nameThisOne.substring(0, 29) + "...";
        thisColumn[n] = nameThisOne;
    }
    return thisColumn;
}

